let MaxRectsPackerEngine = require("maxrects-packer").MaxRectsPacker;

let Packer = require("./Packer");

const METHOD = {
    Smart: "Smart",
    Square: "Square",
    SmartSquare: "SmartSquare"
};

class MaxRectsPacker extends Packer {
    constructor(width, height, allowRotate=false) {
        super();

        this.binWidth = width;
        this.binHeight = height;
        this.allowRotate = allowRotate;
    }

    pack(data, method) {
        let options = {
            smart: (method === METHOD.Smart || method === METHOD.SmartSquare),
            pot: false,
            square: (method === METHOD.Square || method === METHOD.SmartSquare),
            allowRotation: this.allowRotate
        };

        let packer = new MaxRectsPackerEngine(this.binWidth, this.binHeight, 0, options);

        let input = [];
        
        for(let item of data) {
            input.push({width: item.frame.w, height: item.frame.h, data: item});
        }

        packer.addArray(input);
        
        let bin = packer.bins[0];
        let rects = bin.rects;
        
        let res = [];
        
        for(let item of rects) {
            item.data.frame.x = item.x;
            item.data.frame.y = item.y;
            if (item.rot) {
                item.data.rotated = true;
            }
            res.push(item.data);
        }
        
        return res;
    }
    
    static get type() {
        return "MaxRectsPacker";
    }

    static get defaultMethod() {
        return METHOD.Smart;
    }

    static get methods() {
        return METHOD;
    }

    static getMethodProps(id='') {
        switch(id) {
            case METHOD.Smart:
                return {name: "Smart", description: ""};
            case METHOD.Square:
                return {name: "Square", description: ""};
            case METHOD.SmartSquare:
                return {name: "Smart square", description: ""};
            default:
                throw Error("Unknown method " + id);
        }
    }

    static getMethodByType(type) {
        type = type.toLowerCase();

        let keys = Object.keys(METHOD);

        for(let name of keys) {
            if(type === name.toLowerCase()) return METHOD[name];
        }

        return null;
    }
}

module.exports = MaxRectsPacker;